import os

SEQ_RENDERER_TESTS = [
	("Simple sequence",
	':BCF_SEQ_RNDR_INFO_V1:BB10,5,20,1,4:CH1,00,ASDE:ED'),
	("Exo-chain residues",
	':BCF_SEQ_RNDR_INFO_V1:BB10,5,20,3,57:CH0,00,MALW:CH1,00,MFVNQHLCGSHLVEALYLVCGERGFF:CH0,00,YTPKTGSLQ:CH2,00,PLALEGSLQKRGIVEQCN:SS12,56,115,10,115,11,-5,11,-5,27,155,27,155,30:ED'),
	("Modified residues",
	':BCF_SEQ_RNDR_INFO_V1:BB10,5,20,1,8:CH1,00,A2SD1EEF3EF:ED'),
	("Modified terminals and disulfide bridges",
	':BCF_SEQ_RNDR_INFO_V1:BB10,5,20,2,30:CH1,10,ACD1EFCG:CH2,01,QWCRTYIVTIKLWCRDFQWASVK:SS2,10,15,0,15,-2,95,-2,95,0:SS6,21,55,10,55,12,5,12,5,13,5,15:ED'),
	("Fully cyclic chain",
	':BCF_SEQ_RNDR_INFO_V1:BB10,5,20,1,4:CH1,00,ACDC:XL11,1,4,1,5,-1,5,-1,-2,41,-2,41,5,39,5:ED'),
	("Cyclizations",
	':BCF_SEQ_RNDR_INFO_V1:BB10,5,20,1,8:CH1,00,ACDC:CH2,00,ACDC:XL10,1,7,1,5,-1,5,-1,-2,65,-2,65,0:XL01,3,5,25,0,25,-1,41,-1,41,5,43,5:ED')
	]

# Test data for JavaScript, Flash, and Java applets that show live in test web page overview.
js_file = open("seqtest_data.js", 'w')
js_file.write('var testcase_names = [];\n')
js_file.write('var render_infos = [];\n')

i = 0
for testcase_name, render_info in SEQ_RENDERER_TESTS:
	js_file.write('testcase_names[' + str(i) + '] = "' + testcase_name + '";\n')
	js_file.write('render_infos[' + str(i) + '] = "' + render_info + '";\n')
	js_file.write('\n')
	i = i + 1

js_file.close()

# When generating images via external applications, render_info is written to a file and then the file name is passed
# to the application. Passing render info as a command line parameter isn't easily done since the commas will split
# the render info into multiple parameters and spaces will too. Trying to fix it via quoting is close to a nightmare.

# Generate images via the Pascal renderer.
i = 1
for testcase_name, render_info in SEQ_RENDERER_TESTS:
	tmp_file = open('render_info.txt', 'w')
	tmp_file.write(render_info)
	tmp_file.close()
	
	os.system("..\\delphi\\RenderPascalSeqGraphics.exe render_info.txt seq_pascal_image_" + str(i) + ".bmp")
	i = i + 1


# Generate images via the .NET renderer.
i = 1
for testcase_name, render_info in SEQ_RENDERER_TESTS:
	tmp_file = open('render_info.txt', 'w')
	tmp_file.write(render_info)
	tmp_file.close()
	
	os.system("..\\dotnet\\RenderDotNetSeqGraphics\\bin\\Debug\\RenderDotNetSeqGraphics.exe render_info.txt seq_dotnet_image_" + str(i) + ".bmp")
	i = i + 1

os.remove('render_info.txt')